//+------------------------------------------------------------------+
//|   BetterVolumeTicks.mq4
//+------------------------------------------------------------------+
#property indicator_separate_window
#property indicator_buffers 9
#property indicator_color1  clrRed
#property indicator_color2  clrDarkGray
#property indicator_color3  clrYellow
#property indicator_color4  clrLime
#property indicator_color5  clrWhite
#property indicator_color6  clrMagenta
#property indicator_color7  clrLimeGreen
#property indicator_color8  clrRed
#property indicator_color9  clrRed
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property indicator_width4  2
#property indicator_width5  2
#property indicator_width6  2
#property indicator_width7  2
#property indicator_width8  2
#property indicator_width9  2

//
//
//
//
//

enum enMaTypes
{
   ma_sma,    // Simple moving average
   ma_ema,    // Exponential moving average
   ma_smma,   // Smoothed MA
   ma_lwma,   // Linear weighted MA
   ma_tema    // Triple exponential moving average - TEMA
};

extern int       VolPeriod          = 15;
extern enMaTypes VolAvgMethod       = ma_lwma;          // Volume average type
extern int       LookBack           = 7;
extern double    BuySellDistance    = 50;
extern string    note               = "turn on Alert = true; turn off = false";
extern bool      alertsOn           = true;
extern bool      alertsOnCurrent    = true;
extern bool      alertsMessage      = true;
extern bool      alertsSound        = true;
extern bool      alertsEmail        = false;
extern bool      alertsNotify       = false;
extern string    soundfile          = "alert2.wav";
extern bool      ShowLabels         = true;
extern string    labelsIdentifier   = "better volume1";
extern int       fontSize           = 8;
extern int       xDistance          = 10;
extern int       yDistance          = 10;
extern color     LabelsColorUp      = clrLimeGreen;
extern color     LabelsColorNeutral = clrDimGray;
extern color     LabelsColorDown    = clrRed;

//
//
//
//
//

double ClimaxHi[],Neutral[],LoVolume[],Churn[],ClimaxLo[],ClimaxChurn[],AvgVol[],avda[],avdb[],UpTicks[],DnTicks[],VolRange[],VolSort[],trend[],slope[];
string symbol;
int    digits;
string shortName;
double modifier  = 1;


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
//
//

int init()
{
   IndicatorBuffers(15);
   SetIndexBuffer(0, ClimaxHi);   SetIndexStyle(0,DRAW_HISTOGRAM); SetIndexLabel(0,"ClimaxHi");
   SetIndexBuffer(1, Neutral);    SetIndexStyle(1,DRAW_HISTOGRAM); SetIndexLabel(1,"Neutral");
   SetIndexBuffer(2, LoVolume);   SetIndexStyle(2,DRAW_HISTOGRAM); SetIndexLabel(2,"LoVolume");
   SetIndexBuffer(3, Churn);      SetIndexStyle(3,DRAW_HISTOGRAM); SetIndexLabel(3,"Churn");
   SetIndexBuffer(4, ClimaxLo);   SetIndexStyle(4,DRAW_HISTOGRAM); SetIndexLabel(4,"ClimaxLo");
   SetIndexBuffer(5, ClimaxChurn);SetIndexStyle(5,DRAW_HISTOGRAM); SetIndexLabel(5,"ClimaxChurn");
   SetIndexBuffer(6, AvgVol);                                      SetIndexLabel(6,"AvgVolume");
   SetIndexBuffer(7, avda);                                        SetIndexLabel(7,"AvgVolume");
   SetIndexBuffer(8, avdb);                                        SetIndexLabel(8,"AvgVolume");
   SetIndexBuffer(9, UpTicks);  
   SetIndexBuffer(10,DnTicks);  
   SetIndexBuffer(11,VolRange);   
   SetIndexBuffer(12,VolSort);
   SetIndexBuffer(13,trend);  
   SetIndexBuffer(14,slope);                              

   symbol = Symbol(); if (StringSubstr(symbol,0,2)=="_t") symbol = StringSubstr(symbol,2);
   digits = MarketInfo(symbol,MODE_DIGITS);   
	if(digits == 3 || digits == 5) modifier = 10;
	
	shortName = labelsIdentifier+":  Better Volume";
   IndicatorShortName(shortName);   
return(0);
}

//
//
//
//
//

int deinit()
{
   string lookFor       = labelsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   return(0);
}

//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //
   
   if  (slope[limit] == -1) CleanPoint(limit,avda,avdb);
   for(int i = limit; i >= 0; i--)
   {
      VolSort[i]   = Volume[i]/modifier;
      AvgVol[i]    = iCustomMa(VolAvgMethod,VolSort[i],VolPeriod,i);
      double Range = High[i]-Low[i];
      double CountUp = (Volume[i]+(Close[i]-Open[i])/Point)/2;
      double CountDn = Volume[i]-CountUp;
      double diff    = fabs(CountUp-CountDn);
      avda[i]  = EMPTY_VALUE;
      avdb[i]  = EMPTY_VALUE;
      slope[i] = (i<Bars-1) ? (AvgVol[i]>AvgVol[i+1])                   ? 1 : (AvgVol[i]<AvgVol[i+1])                   ? -1 : slope[i+1] : 0;  
      trend[i] = (i<Bars-1) ? (CountUp>CountDn && diff>BuySellDistance) ? 1 : (CountUp<CountDn && diff>BuySellDistance) ? -1 : trend[i+1] : 0;
      if (slope[i] == -1) PlotPoint(i,avda,avdb,AvgVol);  
      

      UpTicks[i] = CountUp*Range;
      DnTicks[i] = CountDn*Range;
      
      if (Range != 0) { VolRange[i] = Volume[i]/Range; }
      
      double LoVol = Volume[iLowest(NULL, 0,MODE_VOLUME,LookBack,i)];
      double HiVol = Volume[iHighest(NULL,0,MODE_VOLUME,LookBack,i)];

      double HiUpTick = UpTicks[FindMaxUp(i)];
      double HiDnTick = DnTicks[FindMaxDn(i)];
      double MaxVol   = VolRange[FindMaxVol(i)];
      
      Neutral[i]     = Volume[i]/modifier;
      ClimaxHi[i]    = EMPTY_VALUE;      
      ClimaxLo[i]    = EMPTY_VALUE;      
      Churn[i]       = EMPTY_VALUE;      
      LoVolume[i]    = EMPTY_VALUE;      
      ClimaxChurn[i] = EMPTY_VALUE;      
      
      if (Volume[i] == LoVol)
      {
         LoVolume[i] = Volume[i]/modifier;
         Neutral[i]  = EMPTY_VALUE;
      }

      if (VolRange[i] == MaxVol)
      {
         Churn[i]    = Volume[i]/modifier;                
         Neutral[i]  = EMPTY_VALUE;
         LoVolume[i] = EMPTY_VALUE;
      }

      if (UpTicks[i] == HiUpTick && Close[i] >= (High[i]+Low[i])*0.5)
      {
         ClimaxHi[i] = Volume[i]/modifier;
         Neutral[i]  = EMPTY_VALUE;
         LoVolume[i] = EMPTY_VALUE;
         Churn[i]    = EMPTY_VALUE;
      }   
         
      if (DnTicks[i] == HiDnTick && Close[i] <= (High[i]+Low[i])*0.5)
      {
         ClimaxLo[i] = Volume[i]/modifier;
         Neutral[i]  = EMPTY_VALUE;
         LoVolume[i] = EMPTY_VALUE;
         Churn[i]    = EMPTY_VALUE;
      }   
         
      if (VolRange[i] == MaxVol && (ClimaxHi[i] != EMPTY_VALUE || ClimaxLo[i] != EMPTY_VALUE))
      {
         ClimaxChurn[i] = Volume[i]/modifier;
         ClimaxHi[i]    = EMPTY_VALUE;
         ClimaxLo[i]    = EMPTY_VALUE;
         Churn[i]       = EMPTY_VALUE;
         Neutral[i]     = EMPTY_VALUE;
       }
        
      }
     
      //
      //
      //
      //
      //
      
      if (alertsOn)
      {
         int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
        if (trend[whichBar] != trend[whichBar+1])
        if (trend[whichBar] == 1)
              doAlert("buyers in control");
        else  doAlert("sellers in control");       
      }
      if (ShowLabels) manageLabels();
return(0);
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

#define _maInstances 1
#define _maWorkBufferx1 1*_maInstances
#define _maWorkBufferx2 2*_maInstances
#define _maWorkBufferx3 3*_maInstances

double iCustomMa(int mode, double price, double length, int r, int instanceNo=0)
{
   int bars = Bars; r = bars-r-1;
   switch (mode)
   {
      case ma_sma   : return(iSma(price,(int)length,r,bars,instanceNo));
      case ma_ema   : return(iEma(price,length,r,bars,instanceNo));
      case ma_smma  : return(iSmma(price,(int)length,r,bars,instanceNo));
      case ma_lwma  : return(iLwma(price,(int)length,r,bars,instanceNo));
      case ma_tema  : return(iTema(price,(int)length,r,bars,instanceNo));
      default       : return(price);
   }
}

//
//
//
//
//

double workSma[][_maWorkBufferx2];
double iSma(double price, int period, int r, int _bars, int instanceNo=0)
{
   if (ArrayRange(workSma,0)!= _bars) ArrayResize(workSma,_bars); instanceNo *= 2; int k;

   workSma[r][instanceNo+0] = price;
   workSma[r][instanceNo+1] = price; for(k=1; k<period && (r-k)>=0; k++) workSma[r][instanceNo+1] += workSma[r-k][instanceNo+0];  
   workSma[r][instanceNo+1] /= 1.0*k;
   return(workSma[r][instanceNo+1]);
}

//
//
//
//
//

double workEma[][_maWorkBufferx1];
double iEma(double price, double period, int r, int _bars, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= _bars) ArrayResize(workEma,_bars);

   workEma[r][instanceNo] = price;
   if (r>0 && period>1)
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+(2.0/(1.0+period))*(price-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
}

//
//
//
//
//

double workSmma[][_maWorkBufferx1];
double iSmma(double price, double period, int r, int _bars, int instanceNo=0)
{
   if (ArrayRange(workSmma,0)!= _bars) ArrayResize(workSmma,_bars);

   workSmma[r][instanceNo] = price;
   if (r>1 && period>1)
          workSmma[r][instanceNo] = workSmma[r-1][instanceNo]+(price-workSmma[r-1][instanceNo])/period;
   return(workSmma[r][instanceNo]);
}

//
//
//
//
//

double workLwma[][_maWorkBufferx1];
double iLwma(double price, double period, int r, int _bars, int instanceNo=0)
{
   if (ArrayRange(workLwma,0)!= _bars) ArrayResize(workLwma,_bars);
   
   workLwma[r][instanceNo] = price; if (period<=1) return(price);
      double sumw = period;
      double sum  = period*price;

      for(int k=1; k<period && (r-k)>=0; k++)
      {
         double weight = period-k;
                sumw  += weight;
                sum   += weight*workLwma[r-k][instanceNo];  
      }             
      return(sum/sumw);
}

//
//
//
//
//

double workTema[][_maWorkBufferx3];
#define _tema1 0
#define _tema2 1
#define _tema3 2

double iTema(double price, double period, int r, int bars, int instanceNo=0)
{
   if (period<=1) return(price);
   if (ArrayRange(workTema,0)!= bars) ArrayResize(workTema,bars); instanceNo*=3;

   //
   //
   //
   //
   //
      
   workTema[r][_tema1+instanceNo] = price;
   workTema[r][_tema2+instanceNo] = price;
   workTema[r][_tema3+instanceNo] = price;
   double alpha = 2.0 / (1.0+period);
   if (r>0)
   {
          workTema[r][_tema1+instanceNo] = workTema[r-1][_tema1+instanceNo]+alpha*(price                         -workTema[r-1][_tema1+instanceNo]);
          workTema[r][_tema2+instanceNo] = workTema[r-1][_tema2+instanceNo]+alpha*(workTema[r][_tema1+instanceNo]-workTema[r-1][_tema2+instanceNo]);
          workTema[r][_tema3+instanceNo] = workTema[r-1][_tema3+instanceNo]+alpha*(workTema[r][_tema2+instanceNo]-workTema[r-1][_tema3+instanceNo]); }
   return(workTema[r][_tema3+instanceNo]+3.0*(workTema[r][_tema1+instanceNo]-workTema[r][_tema2+instanceNo]));
}

//
//
//
//
//

int FindMaxUp(int i)      
{
   int x,y;
   double max=0;
   for(x=LookBack-1;x>=0;x--) { if(UpTicks[i+x] > max) { y = i+x; max = UpTicks[y];  } }
return(y);
}

//
//
//
//
//

int FindMaxDn(int i)      
{
   int x,y;
   double max=0;
   for(x=LookBack-1;x>=0;x--) { if(DnTicks[i+x] > max) { y = i+x; max = DnTicks[y]; } }
return(y);
}

//
//
//
//
//

int FindMaxVol(int i)      
{
   int x,y;
   double max=0;
   for(x=LookBack-1;x>=0;x--)  { if(VolRange[i+x] > max) { y = i+x; max = VolRange[y]; }}
return(y);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i]; first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] = from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                          second[i] = EMPTY_VALUE; }
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Better Volume ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Better Volume "),message);
             if (alertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," Better Volume " +" "+message));
             if (alertsSound)   PlaySound(soundfile);
      }
}
      
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageLabels()
{
   int window = WindowFind(shortName);
   double ut = (Volume[0]+(Close[0]-Open[0])/Point) / 2.0;
   double dt = (Volume[0] - ut);
   color theColor = LabelsColorNeutral;
      if (ut > dt) theColor = LabelsColorUp;
      if (ut < dt) theColor = LabelsColorDown;
   
   //
   //
   //
   //
   //
      
   string name = labelsIdentifier+":"+"label_bv";
         if(ObjectFind (name) == -1)
            ObjectCreate(name, OBJ_LABEL, window, 0, 0);
               ObjectSet(name, OBJPROP_XDISTANCE, xDistance);
               ObjectSet(name, OBJPROP_YDISTANCE, yDistance);
               ObjectSet(name, OBJPROP_CORNER, 1);
               ObjectSetText(name,"Buyers : " + DoubleToStr(ut,0), fontSize, "Arial Black", theColor);

      name = labelsIdentifier+":"+"label_sv";
         if(ObjectFind (name) == -1)
            ObjectCreate(name, OBJ_LABEL, window, 0, 0);
               ObjectSet(name, OBJPROP_XDISTANCE, xDistance);
               ObjectSet(name, OBJPROP_YDISTANCE, yDistance+14);
               ObjectSet(name, OBJPROP_CORNER, 1);
               ObjectSetText(name,"Sellers : " + DoubleToStr(dt,0), fontSize, "Arial Black", theColor);

}
      
                  